package org.jamesii.experimentgeneration.controller.generation.controller;


import com.scalified.tree.TreeNode;
import javafx.scene.control.TextArea;
import org.everit.json.schema.Schema;
import org.everit.json.schema.ValidationException;
import org.everit.json.schema.loader.SchemaLoader;
import org.jamesii.experimentgeneration.model.BackendType;

import org.jamesii.experimentgeneration.model.ExperimentModel;
import org.json.JSONObject;
import org.json.JSONTokener;


import java.io.*;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.Observable;
import java.util.Observer;

/**
 * Created by Marcus on 17.12.2018.
 */
public class ParsingController implements Observer {

    MainExperimentController mainExperimentController;

    public ParsingController(MainExperimentController mainExperimentController){
        this.mainExperimentController = mainExperimentController;

    }

    protected JSONObject getExperimentParameter(File jsonSchema, File jsonInputs) throws IOException {
        String schemaName = jsonSchema.getName();
        String jsonFileName = jsonInputs.getName();


        try(InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream(schemaName)){
            JSONObject rawSchema = new JSONObject(new JSONTokener(inputStream));

            InputStream jsonFile = this.getClass().getClassLoader().getResourceAsStream(jsonFileName);
            JSONObject jsonObject = new JSONObject((new JSONTokener(jsonFile)));

            schemaCheckExperimentParameter(rawSchema, jsonObject);

            return jsonObject;
        } catch (IOException e) {
            e.printStackTrace();
        }
       return null;
    }

    /**
     * Validates the JSONObject based on the selected schema
     * @param rawSchema
     * @param jsonObject
     */
    public boolean schemaCheckExperimentParameter(JSONObject rawSchema, JSONObject jsonObject) {
        Schema schema = SchemaLoader.load(rawSchema);
        TreeNode<String> experimentRoot = mainExperimentController.getExperimentModel().getExperimentRoot();
        System.out.println(experimentRoot.toString());
        TextArea logger = mainExperimentController.getView().getLogger();
        try{
            schema.validate(jsonObject);
            mainExperimentController.getView().getLogger().setText("Schema Checks Passed");
            System.out.println("Schema checks passed");
        }catch (ValidationException e){
            String s ="";
            s = s+ e.getErrorMessage() + "\n";

            for (String message : e.getAllMessages()){
                    s = s + message + "\n";
            }
            logger.setText(s);
            return false;
        }
        return true;
    }

    public File getFile(String fileName){
        ClassLoader classLoader = getClass().getClassLoader();
        return new File(classLoader.getResource(fileName).getFile());
    }

    protected BackendType identifyBackend(JSONObject jsonObject){
        switch (jsonObject.get("Choose Backend").toString()){
            case "Python":{
                return BackendType.PYTHON;
            }
            case "SESSL/ML-Rules":{
                return BackendType.SESSL;
            }
            case "EMStimTools/YAML":
                return BackendType.YAML;
        }
        return null;
    }

    @Override
    public void update(Observable o, Object arg) {

    }
}
