package org.jamesii.experimentgeneration.controller.generation.controller.backend;


import org.apache.commons.io.FilenameUtils;
import org.jamesii.experimentgeneration.controller.generation.controller.MainExperimentController;
import org.jamesii.experimentgeneration.experiment.Experiment;
import org.jamesii.experimentgeneration.model.ExperimentModel;
import org.json.JSONArray;
import org.json.JSONObject;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;

/**
 * Created by Marcus on 20.02.2019.
 */
public abstract class BackendController implements Experiment {
    private MainExperimentController mainExperimentController;

    public BackendController(MainExperimentController mainExperimentController) {
        this.mainExperimentController = mainExperimentController;
    }

    protected boolean checkFile(JSONObject jsonObject) {
        String modelPath = jsonObject.get("model_path").toString();


        boolean fileCheck = checkFileExistence(modelPath);
        boolean fileExtension = checkFileExtension(modelPath);

        return fileCheck && fileExtension;
    }

    protected boolean checkFileExtension(String filePath) {
        File file = new File(filePath);
        String extension = FilenameUtils.getExtension(file.getName());
        if (extension.equals("mlrj")) {
            System.out.println("File Extension : " + extension + " valid");
            return true;
        } else {
            System.out.println("Invalid File Extension : " + extension);
            return false;
        }
    }

    protected boolean checkFileExistence(String filePath) {
        try {
            File file = new File((filePath));
            boolean exists = file.getAbsoluteFile().exists();
            if (exists) {
                System.out.println("File Existence checked");
                return true;
            } else {
                System.out.println("Incorrect File Path, File does not Exist");
                return false;
            }
        } catch (NullPointerException e) {
            e.printStackTrace();
        }
        return false;
    }


    protected void adaptModelPath(ExperimentModel experimentModel) {
        String modelPath = experimentModel.getExperimentSpecification().get("modelPath").toString();
        String[] quickstarts = modelPath.split("quickstart");
        experimentModel.getExperimentSpecification().put("modelPath", "." + quickstarts[1]);
    }

//
//    private void addHashToExperimentParameter(JSONObject jsonProperty, String key) {
//        HashMap experimentParameter = mainExperimentController.getExperimentModel().getExperimentParameter();
//        ArrayList<String> hashList = new ArrayList<>();
//        for (String key2 : jsonProperty.keySet()) {
//
//            ArrayList<String> propertyItems = new ArrayList();
//            for (Object arrayItem : ((JSONArray) jsonProperty.get(key2))) {
//                propertyItems.add(arrayItem.toString());
//                experimentParameter.put(key2, propertyItems);
//            }
//            hashList.add(key2);
//
//        }
//        String hashKeyList = hashList.get(0);
//        String hashValueList = hashList.get(1);
//
//        assembleMap(hashKeyList, hashValueList, key);
//
//
//    }
//
//    private void assembleMap(String hashKey1, String hashKey2, String key) {
//        HashMap experimentParameter = mainExperimentController.getExperimentModel().getExperimentParameter();
//        ArrayList keyList = (ArrayList) experimentParameter.get(hashKey1);
//        ArrayList valueList = (ArrayList) experimentParameter.get(hashKey2);
//
//        if (keyList != null && valueList != null) {
//            experimentParameter.remove(hashKey1);
//            experimentParameter.remove(hashKey2);
//            HashMap zippedMap = zipLists(keyList, valueList);
//            experimentParameter.put(key, zippedMap);
//        }
//    }
//
//    private HashMap zipLists(ArrayList<String> list1, ArrayList<String> list2) {
//        HashMap hashMap = new HashMap();
//        if (list1.size() == list2.size()) {
//            for (int i = 0; i < list1.size(); i++) {
//                String hashKey = list1.get(i);
//                String hashValue = list2.get(i);
//                hashMap.put(hashKey, hashValue);
//            }
//        }
//        return hashMap;
//    }

    public void extractExperimentParameter(HashMap experimentParameter, JSONObject jsonObject) {
        ExperimentModel experimentModel = mainExperimentController.getExperimentModel();

        LinkedHashMap<String, ArrayList> zipInstances = experimentModel.getZipInstances();
        for (String key : jsonObject.keySet()) {
            Object jsonProperty = jsonObject.get(key);
            //If Property is Json Object
            if (jsonProperty instanceof JSONObject) {
                if (zipInstances.containsKey(key) && zipInstances.get(key).size() >= 2) {
                    for (String objectKey : ((JSONObject) jsonProperty).keySet()) {
                        Object subProperty = ((JSONObject) jsonProperty).get(objectKey);
                        if (subProperty instanceof JSONArray) {
                            ArrayList list;
                            list = (ArrayList) ((JSONArray) subProperty).toList();
                            experimentParameter.put(objectKey, list);
                        }

                    }
                } else {
                    extractExperimentParameter(experimentParameter, (JSONObject) jsonProperty);
                }
            }
            //If Property is JsonArray
            else if (jsonProperty instanceof JSONArray) {
                ArrayList<String> propertyItems = new ArrayList();
                for (Object arrayItem : ((JSONArray) jsonProperty)) {
                    propertyItems.add(arrayItem.toString());
                    experimentParameter.put(key, propertyItems);
                }
            } else {
                experimentParameter.put(key, jsonProperty.toString());
            }
        }
    }


}
