package org.jamesii.experimentgeneration.view;

import javafx.geometry.Insets;
import javafx.scene.control.*;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import org.jamesii.experimentgeneration.model.ExperimentModel;
import org.jamesii.experimentgeneration.model.ExperimentType;
import org.json.JSONObject;

import java.io.*;
import java.util.LinkedHashMap;

/**
 * Created by Marcus on 02.08.2019.
 */
public class ExperimentTab extends Tab {
//    TreeView<File> tree;
    View view;
    private ExperimentModel experimentModel;
    private BorderPane borderLayout;
    private Label selectExperimentDesign;
    private CheckBox experimentDesign;
    private ComboBox experimentType;
    private Label selectExperimentType;
    private GridPane gridPane;

    public ExperimentTab(ExperimentModel experimentModel, View view) {
        this.experimentModel = experimentModel;
        this.setText("Schema");
        this.setClosable(false);
        this.gridPane = new GridPane();
        this.experimentDesign = new CheckBox("");
        this.selectExperimentType = new Label("Select Type of Simulation");
        this.selectExperimentDesign = new Label("Add Experiment Design (optional)");
        this.experimentType = new ComboBox();
        borderLayout = new BorderPane();


        this.view = view;
        setup();
    }

    public CheckBox getExperimentDesign() {
        return experimentDesign;
    }

    private void setup() {
    	experimentType.getItems().add("Stochastic Discrete-Event");
        experimentType.getItems().add("Finite Element Method");
        experimentType.setOnAction(event -> {
            if (experimentType.getSelectionModel().getSelectedItem().toString().equals("Finite Element Method")) {
                generateElectromagneticExperiment();
            } else if (experimentType.getSelectionModel().getSelectedItem().toString().equals("Stochastic Discrete-Event")) {
                generateStochasticExperiment();
            }
        });
        experimentDesign.setDisable(true);
        this.experimentDesign.setOnAction(event -> {
            handleExperimentDesign();
        });
        gridPane.add(selectExperimentType, 0, 0);
        gridPane.add(experimentType, 1, 0);
        gridPane.add(selectExperimentDesign, 0, 1);
        gridPane.add(experimentDesign, 1, 1);
        gridPane.setPadding(new Insets(5, 5, 5, 5));
        gridPane.setHgap(10);
        borderLayout.setCenter(gridPane);
        this.setContent(borderLayout);

    }

    private void generateStochasticExperiment() {
        experimentModel.clearData();
        experimentModel.setExperimentType(ExperimentType.STOCHASTIC_DISCRETE_EVENT);
        //Reset View
        view.clearView();
        InputStream inputStream = experimentModel.accessFile("schemas/stochasticDiscreteEventSchema.json");
        File schemaFile = experimentModel.getJarFile(inputStream, "schemas/stochasticDiscreteEventSchema.json");


        LinkedHashMap linkedHashMap = experimentModel.getJsonAsLinkedHashMap(schemaFile);
        experimentModel.setSchemaFile(new JSONObject(linkedHashMap));
        view.generateInputElements(schemaFile);

        if (experimentDesign.isSelected()) {
            handleExperimentDesign();
        }
    }



    private void generateElectromagneticExperiment() {
        experimentModel.clearData();
        experimentModel.setExperimentType(ExperimentType.FINITE_ELEMENT);
        //Reset View
        view.clearView();
        InputStream inputStream = experimentModel.accessFile("schemas/electromagneticExperimentSchema.json");
        File schemaFile = experimentModel.getJarFile(inputStream, "schemas/electromagneticExperimentSchema.json");

        LinkedHashMap linkedHashMap = experimentModel.getJsonAsLinkedHashMap(schemaFile);
        experimentModel.setSchemaFile(new JSONObject(linkedHashMap));
        view.generateInputElements(schemaFile);
        if (experimentDesign.isSelected()) {
            handleExperimentDesign();
        }
    }

    private void handleExperimentDesign() {
        if (experimentDesign.isSelected()) {
            addExperimentDesign();
        } else if (!experimentDesign.isSelected()) {
            experimentModel.removeExperimentDesign(view);
        }
    }

    private void addExperimentDesign() {
        InputStream inputStream = experimentModel.accessFile("schemas/experimentDesignSchema.json");
        File schemaFile = experimentModel.getJarFile(inputStream, "schemas/experimentDesignSchema.json");

        view.generateInputElements(schemaFile);
    }

    private TreeItem<File> createTree(File file) {
        TreeItem<File> item = new TreeItem<>(file);

        File[] childs = file.listFiles();
        if (childs != null) {
            for (File child : childs) {
                item.getChildren().add(createTree(child));
            }
        }
        return item;
    }
}
